/*
 *******************************************************************************
 *  Copyright (c) 2010-2022 VATICS(KNERON) Inc. All rights reserved.
 *
 *  +-----------------------------------------------------------------+
 *  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
 *  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
 *  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
 *  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
 *  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
 *  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
 *  |                                                                 |
 *  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
 *  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
 *  | VATICS(KNERON) INC.                                             |
 *  +-----------------------------------------------------------------+
 *
 *******************************************************************************
 */
#ifndef _LVGL_WIDGETS_H_
#define _LVGL_WIDGETS_H_

void lv_ex_clean_display(void);
void lv_ex_demo(void);
void lv_ex_font(void);
void lv_ex_win_1(void);
void lv_ex_chart_1(void);
void lv_ex_ddlist_1(void);
void lv_ex_calendar_1(void);
void lv_ex_roller_1(void);
void lv_ex_preload_1(void);
void lv_ex_page_1(void);
void lv_ex_mbox_1(void);
void lv_ex_lmeter_1(void);
void lv_ex_list_1(void);
void lv_ex_line_1(void);
void lv_ex_led_1(void);
void lv_ex_label_1(void);
void lv_ex_kb_1(void);
void lv_ex_gauge_1(void);
void lv_ex_cont_1(void);
void lv_ex_cb_1(void);
void lv_ex_btnm_1(void);
void lv_ex_btn_1(void);
void lv_ex_bar_1(void);
void lv_ex_arc_1(void);
void lv_ex_slider_1(void);
void lv_ex_spinbox_1(void);
void lv_ex_sw_1(void);
void lv_ex_table_1(void);
void lv_ex_tabview_1(void);
void lv_ex_tileview_1(void);
void lv_ex_color_picker_1(void);
void lv_ex_canvas_1(void);
void lv_ex_ta_1(void);
void lv_ex_img_1(void);
void lv_ex_imgbtn_1(void);
void lv_ex_group_keyboard_slider_1(void);
void lv_ex_img_2(void);
void lv_ex_animation_1(void);
void lv_ex_img_3(void);

#endif //_LVGL_WIDGETS_H_
